@@ -44,9 +44,27 @@ static int seq_open(struct seq_state *state, struct seq_conn *conn, bool with_qu
 
 #ifdef HAVE_ALSA_UMP
 	res = snd_seq_set_client_midi_version(conn->hndl, SND_SEQ_CLIENT_UMP_MIDI_2_0);
+	if (!res) {
+		snd_seq_client_info_t *info = NULL;
+
+		/* Double check client version */
+		res = snd_seq_client_info_malloc(&info);
+		if (!res)
+			res = snd_seq_get_client_info(conn->hndl, info);
+		if (!res) {
+			res = snd_seq_client_info_get_midi_version(info);
+			if (res == SND_SEQ_CLIENT_UMP_MIDI_2_0)
+				res = 0;
+			else
+				res = -EIO;
+		}
+		if (info)
+			snd_seq_client_info_free(info);
+	}
 #else
 	res = -EOPNOTSUPP;
 #endif
+
 	if (res < 0) {
 		spa_log_lev(state->log, (probe_ump ? SPA_LOG_LEVEL_INFO : SPA_LOG_LEVEL_ERROR),
 				"%p: ALSA failed to enable UMP MIDI: %s", state, snd_strerror(res));
@@ -60,7 +78,6 @@ static int seq_open(struct seq_state *state, struct seq_conn *conn, bool with_qu
 		spa_log_debug(state->log, "%p: ALSA UMP MIDI enabled", state);
 		state->ump = true;
 	}
-
 	return 0;
 }
 
@@ -386,7 +403,6 @@ int spa_alsa_seq_open(struct seq_state *state)
 
 	state->sys.source.func = alsa_seq_on_sys;
 	state->sys.source.data = state;
-	spa_loop_add_source(state->main_loop, &state->sys.source);
 
 	/* increase event queue timer resolution */
 	snd_seq_queue_timer_alloca(&timer);
@@ -431,6 +447,8 @@ int spa_alsa_seq_open(struct seq_state *state)
 
 	state->timerfd = res;
 
+	spa_loop_add_source(state->main_loop, &state->sys.source);
+
 	state->opened = true;
 
 	return 0;
@@ -802,6 +820,7 @@ static int process_write(struct seq_state *state)
 		struct spa_pod_control *c;
 		uint64_t out_time;
 		snd_seq_real_time_t out_rt;
+		bool first = true;
 
 		if (!port->valid || io == NULL)
 			continue;
@@ -866,13 +885,20 @@ static int process_write(struct seq_state *state)
 				if ((size = spa_ump_to_midi((uint32_t *)body, body_size, data, sizeof(data))) <= 0)
 					continue;
 
-				snd_seq_ev_clear(&ev);
+				if (first)
+					snd_seq_ev_clear(&ev);
 
-				snd_midi_event_reset_encode(stream->codec);
-				if ((size = snd_midi_event_encode(stream->codec, data, size, &ev)) <= 0) {
+				if ((size = snd_midi_event_encode(stream->codec, data, size, &ev)) < 0) {
 					spa_log_warn(state->log, "failed to encode event: %s", snd_strerror(size));
+					snd_midi_event_reset_encode(stream->codec);
+					first = true;
 					continue;
 				}
+				first = false;
+				if (ev.type == SND_SEQ_EVENT_NONE)
+					/* this can happen when the event is not complete yet, like
+					 * a sysex message and we need to encode some more data. */
+					continue;
 
 				snd_seq_ev_set_source(&ev, state->event.addr.port);
 				snd_seq_ev_set_dest(&ev, port->addr.client, port->addr.port);
@@ -882,6 +908,7 @@ static int process_write(struct seq_state *state)
 					spa_log_warn(state->log, "failed to output event: %s",
 							snd_strerror(err));
 				}
+				first = true;
 			}
 		}
 	}
